<?php

/**
 * @file
 *   Translation template extractor module drush integration.
 */

/**
 * Implements hook_drush_command().
 *
 * @see drush_parse_command() for a list of recognized keys.
 *
 * @return
 *   An associative array describing our command.
 */
function potx_drush_command() {
  $items['potx'] = array(
    'callback' => 'potx_drush_extract',
    'description' => 'Extract translatable strings from Drupal source code.',
    'options' => array(
      'modules' => 'Comma delimited list of modules to extract translatable strings from.',
      'files' => 'Comma delimited list of files to extract translatable strings from.',
      'folder' => 'Folder to begin translation extraction in. When no other option is set this defaults to current directory.',
      'api' => 'Drupal core version to use for extraction settings.',
    ),
    'examples' => array(
      'potx' => 'Extract translatable strings from applicable files in current directory and write to output file',
      'potx --modules=example' => 'Extract translatable strings from applicable files of example module and write to output file.',
      'potx --files=sites/all/modules/example/example.module' => 'Extract translatable strings from example.module and write to output file.',
      'potx --api=8 --folder=projects/drupal/8' => 'Extract strings from folder projects/drupal/8 using API version 8.',
    ),
  );
  return $items;
}

/**
 * Implementation of hook_drush_help().
 *
 * This function is called whenever a drush user calls
 * 'drush help potx'.
 *
 * @param
 *   A string with the help section (prepended with 'drush:').
 *
 * @return
 *   A string with the help text for our command.
 */
function potx_drush_help($section) {
  if ($section == 'drush:potx') {
    $help = dt('Generates translation templates from the given Drupal source code in the current working directory.');
    $help .= "\n\n". dt('If no files are specified, potx will autodiscover files from the current working directory. You can specify concrete files to look at to limit the scope of the operation.');
    return $help;
  }
}

/**
 * Drush command callback.
 */
function potx_drush_extract() {
  // Include library.
  include_once dirname(__FILE__) . '/potx.inc';

  // Get Drush options.
  $modules_option = drush_get_option('modules');
  $files_option = drush_get_option('files');
  $folder_option = drush_get_option('folder');
  $api_option = drush_get_option('api');
  if (empty($api_option) || !in_array($api_option, array(5, 6, 7, 8))) {
    $api_option = POTX_API_CURRENT;
  }

  if (!empty($modules_option)) {
    $modules = explode(',', $modules_option);
    foreach ($modules as $module) {
      $files = array_merge($files, potx_parser_source_files(drupal_get_path('module', $module), '*', $api_option));
    }
  }
  elseif (!empty($files_option)) {
    $files = explode(',', $files_option);
  }
  elseif (!empty($folder_option)) {
    $files = potx_parser_source_files($folder_option, '*', $api_option);
  }
  else {
    // No file list provided so autodiscover files in current directory.
    $files = potx_parser_source_files(drush_cwd(), '*', $api_option);
  }

  foreach ($files as $file) {
    drush_print("Processing $file...");
    potx_parser_parse_file($file, 0, 'potx_parser_callback_save_string', 'potx_parser_callback_version', $api_option);
  }

  $output = potx_parser_build_output(POTX_STRING_RUNTIME, 'potx_parser_callback_save_string', 'potx_parser_callback_version', 'potx_parser_callback_header', NULL, NULL, $api_option);
  potx_drush_write_file($output);

  drush_print("");
  drush_print(dt("Stats"));
  $header = array(
      'files' => dt('Files'),
      'strings' => dt('Strings'),
      'warnings' => dt('Warnings'),
  );
  $rows = array(array_values($header));
  // Get errors, if any.
  $errors = potx_parser_util_get_messages();
  // Get saved strings.
  $strings  = potx_parser_callback_save_string(NULL, NULL, NULL, 0, POTX_STRING_RUNTIME);
  $rows[] = array(count($files), count($strings), count($errors));
  drush_print_table($rows, TRUE);

  if (!empty($errors)) {
    drush_print(dt("Errors"));
    foreach ($errors as $error) {
      drush_set_error($error[5]);
    }
  }

  drush_print("");
  drush_print(dt("Done"));
}


/**
 * Writes a file with given strings.
 *
 * @param string $file_name
 *   Target file name.
 * @param string $output
 *   The string that will be written in the file.
 */
function potx_drush_write_file($output) {
  $fp = fopen('general.pot', 'w');
  $fw = fwrite($fp, $output);
  $fc = fclose($fp);
}
